@@ -38,7 +38,9 @@ static char rcsid[] = "$Id: arg.c 900 2008-01-05 01:13:26Z hubert@u.washington.e
 int  process_debug_str(char *);
 void args_add_attach(PATMT **, char *, int);
 int  pinerc_cmdline_opt(char *);
+void display_config_options(char *, char *);
 
+extern char configoptions[];
 
 /*
  * Name started as to invoke function key mode
@@ -83,7 +85,9 @@ static char args_err_missing_copyabook[] =	N_("missing argument for option \"-co
 static char args_err_missing_server_name[] =	N_("missing server name. Example: -xoauth2-server Gmail");
 static char args_err_missing_client_id[] =	N_("missing client-id name. Example: -xoauth2-client-id 760.someserver.com");
 static char args_err_missing_client_secret[] =	N_("missing client-secret name. Example: -xoauth2-client-secret V56i0fa_");
-static char args_err_missing_xoauth_option[] =  N_("at least one of the arguments -xoauth2-server, or -xoauth2-client-id, or -xoauth2-client-secret is missing.");
+static char args_err_missing_tenant[] =	N_("missing tenant value. Example: -xoauth2-tenant common");
+static char args_err_missing_user[] =	N_("missing username value. Example: -xoauth2-user user@example.com");
+static char args_err_missing_xoauth_option[] =  N_("at least one of the arguments -xoauth2-server, or -xoauth2-client-id, xoauth2-tenant, or -xoauth2-client-secret is missing.");
 
 static char *args_pine_args[] = {
 N_("Possible Starting Arguments for Alpine program:"),
@@ -190,7 +194,68 @@ N_("\tNote: All of -xoauth options above must be used, if any of them is used"),
 NULL
 };
 
+void
+display_config_options(char *configopts, char *defprefix)
+{
+  char *copts = configopts ? cpystr(configopts) : NULL;
+  char **options = NULL;
+  char *s, *t, *prefix;
+  int len, nlines, pending;
 
+  for(nlines = 0, s = copts; s != NULL;  nlines++){
+     s = strchr(s, '\'');
+     if(s != NULL) s++;
+  }
+  options = fs_get((nlines/2 + 4)*sizeof(char *));	/* 4 in worst case scenario */
+  memset((void *) options, 0, (nlines/2 + 4)*sizeof(char *));
+  for(s = copts, len = 0, nlines = 0; s != NULL; ){
+     pending = 0;
+     prefix = len == 0 ? (nlines == 0 ? defprefix : "  ") : " ";
+     if((s = strchr(s, '\'')) != NULL){
+	t = strchr(s+1, '\'');
+	*t = '\0';
+	if(!strncmp(s+1, "CFLAGS=", 7)
+	    || !strncmp(s+1, "CC=", 3)
+	    || !strncmp(s+1, "LIBS=", 5)
+	    || !strncmp(s+1, "CPP=", 4)
+	    || !strncmp(s+1, "CPPFLAGS=", 9)
+	    || !strncmp(s+1, "LT_SYS_LIBRARY_PATH=", 20)
+	    || !strncmp(s+1, "LDFLAGS=", 8)){
+	   display_args_err(s+1, NULL, 0);
+	   *t++ = '\'';
+	   s = t;
+	   continue;
+	}
+	if(len + strlen(prefix) + strlen(s+1) > 74 ){
+	   if(len == 0){
+	      options[nlines] = fs_get((strlen(prefix) + strlen(s+1) + 3)*sizeof(char));
+	      sprintf(options[nlines++], "%s%s \\", prefix, s+1);
+	   }
+	   else{
+	      strcat(options[nlines++], " \\");
+	      pending++;
+	   }
+	   len = 0;
+	}
+	else{
+	   if(len == 0){
+	      options[nlines] = fs_get(76*sizeof(char));
+	      *options[nlines] = '\0';
+	   }
+	   strcat(options[nlines], prefix);
+	   strcat(options[nlines], s+1);
+	   len = strlen(options[nlines]);
+	}
+	if(t != NULL)
+	   *t = '\'';
+	if(!pending)
+	   s = t ? t + 1 : NULL;
+     }
+  }
+  display_args_err(NULL, options, 0);
+  free_list_array(&options);
+  fs_give((void **) &copts);
+}
 
 /*
  *  Parse the command line args.
@@ -217,9 +282,7 @@ pine_args(struct pine *pine_state, int argc, char **argv, ARGDATA_S *args)
     char *sort                = NULL;
     char *pinerc_file         = NULL;
     char *lc		      = NULL;
-    char *xoauth2_server      = NULL;
-    char *xoauth2_client_id   = NULL;
-    char *xoauth2_client_secret = NULL;
+    XOAUTH2_INFO_S x;
     int   do_help             = 0;
     int   do_conf             = 0;
     int   usage               = 0;
@@ -232,6 +295,7 @@ pine_args(struct pine *pine_state, int argc, char **argv, ARGDATA_S *args)
     av = argv;
     memset(args, 0, sizeof(ARGDATA_S));
     args->action = aaFolder;
+    memset((void *) &x, 0, sizeof(XOAUTH2_INFO_S));
 
     pine_state->pine_name = (lc = last_cmpnt(argv[0])) ? lc : (lc = argv[0]);
 #ifdef	DOS
@@ -556,9 +620,9 @@ Loop: while(--ac > 0)
 	      else if(strcmp(*av, "xoauth2-server") == 0){
 		  if(--ac){
 		      if((str = *++av) != NULL){
-			  if(xoauth2_server)
-			     fs_give((void **) &xoauth2_server);
-			  xoauth2_server = cpystr(str);
+			  if(x.name)
+			     fs_give((void **) &x.name);
+			  x.name = cpystr(str);
 		      }
 		  }
 		  else{
@@ -570,9 +634,9 @@ Loop: while(--ac > 0)
 	      else if(strcmp(*av, "xoauth2-client-id") == 0){
 		  if(--ac){
 		      if((str = *++av) != NULL){
-			  if(xoauth2_client_id)
-			     fs_give((void **) &xoauth2_client_id);
-			  xoauth2_client_id = cpystr(str);
+			  if(x.client_id)
+			     fs_give((void **) &x.client_id);
+			  x.client_id = cpystr(str);
 		      }
 		  }
 		  else{
@@ -584,9 +648,9 @@ Loop: while(--ac > 0)
 	      else if(strcmp(*av, "xoauth2-client-secret") == 0){
 		  if(--ac){
 		      if((str = *++av) != NULL){
-			  if(xoauth2_client_secret)
-			     fs_give((void **) &xoauth2_client_secret);
-			  xoauth2_client_secret = cpystr(str);
+			  if(x.client_secret)
+			     fs_give((void **) &x.client_secret);
+			  x.client_secret = cpystr(str);
 		      }
 		  }
 		  else{
@@ -595,6 +659,34 @@ Loop: while(--ac > 0)
 		  }
 		  goto Loop;
 	      }
+	      else if(strcmp(*av, "xoauth2-tenant") == 0){
+		  if(--ac){
+		      if((str = *++av) != NULL){
+			  if(x.tenant)
+			     fs_give((void **) &x.tenant);
+			  x.tenant = cpystr(str);
+		      }
+		  }
+		  else{
+		      display_args_err(_(args_err_missing_tenant), NULL, 1);
+		      ++usage;
+		  }
+		  goto Loop;
+	      }
+	      else if(strcmp(*av, "xoauth2-user") == 0){
+		  if(--ac){
+		      if((str = *++av) != NULL){
+			  if(x.users)
+			     fs_give((void **) &x.users);
+			  x.users = cpystr(str);
+		      }
+		  }
+		  else{
+		      display_args_err(_(args_err_missing_user), NULL, 1);
+		      ++usage;
+		  }
+		  goto Loop;
+	      }
 #ifdef	_WINDOWS
 	      else if(strcmp(*av, "install") == 0){
 		  pine_state->install_flag = 1;
@@ -942,21 +1034,15 @@ Loop: while(--ac > 0)
 	exit(-1);
     }
 
-    if((xoauth2_server || xoauth2_client_id || xoauth2_client_secret)
-	&& !(xoauth2_server && xoauth2_client_id && xoauth2_client_secret)){
-	display_args_err(_(args_err_missing_xoauth_option), NULL, 1);
-	++usage;
-    }
-
     if(do_help || usage)
       args_help(); 
 
     if(usage)
       exit(-1);
 
-    if(xoauth2_server){
+    if(x.name){
       char *tmp1, *tmp2;
-      tmp2 = xoauth_config_line(xoauth2_server, xoauth2_client_id, xoauth2_client_secret);
+      tmp2 = xoauth_config_line(&x);
       if(tmp2){
         tmp1 = fs_get((strlen(ps_global->vars[V_XOAUTH2_INFO].name) + strlen(tmp2) + 2)*sizeof(char));
 	if(tmp1){
@@ -979,6 +1065,10 @@ Loop: while(--ac > 0)
 		 datestamp, hoststamp);
 	tmp_20k_buf[SIZEOF_20KBUF-1] = '\0';
 	display_args_err(tmp_20k_buf, NULL, 0);
+	if(*configoptions){
+	  display_args_err(_("Alpine was built with the following options:"), NULL, 0);
+	  display_config_options(configoptions, "./configure ");
+	}
 	exit(0);
     }
 
@@ -1142,6 +1232,7 @@ process_debug_str(char *debug_str)
 	}
     }
 
+    if(usage >= 0) ps_global->write_debug_file++;
     if(!new_style_debug_arg){
 #ifdef CSRIMALLOC
 	ps_global->debug_malloc =
