@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: smime.c 1074 2008-06-04 00:08:43Z hubert@u.washingto
 
 /*
  * ========================================================================
- * Copyright 2013-2017 Eduardo Chappa
+ * Copyright 2013-2020 Eduardo Chappa
  * Copyright 2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -36,8 +36,10 @@ static char rcsid[] = "$Id: smime.c 1074 2008-06-04 00:08:43Z hubert@u.washingto
 #include "../pith/list.h"
 #include "../pith/mailcmd.h"
 #include "../pith/tempfile.h"
+#include "../pith/body.h"
 #include "radio.h"
 #include "keymenu.h"
+#include "mailcmd.h"
 #include "mailview.h"
 #include "conftype.h"
 #include "confscroll.h"
@@ -115,8 +117,8 @@ smime_check(BODY *body)
   }
   if(rv > 0) return rv;
   if(body->sparep)
-	p7 = get_smime_sparep_type(body->sparep) == P7Type
-		? (PKCS7 *)get_smime_sparep_data(body->sparep)
+	p7 = get_body_sparep_type(body->sparep) == P7Type
+		? (PKCS7 *)get_body_sparep_data(body->sparep)
 		: NULL;
   if(p7 && (PKCS7_type_is_signed(p7) || PKCS7_type_is_enveloped(p7)))
     rv += 1;
@@ -236,8 +238,8 @@ format_smime_info(int pass, BODY *body, long msgno, gf_io_t pc)
           format_smime_info(pass, &p->body, msgno, pc);
     }
     if(body->sparep)
-       p7 = get_smime_sparep_type(body->sparep) == P7Type 
-		? (PKCS7 *)get_smime_sparep_data(body->sparep)
+       p7 = get_body_sparep_type(body->sparep) == P7Type 
+		? (PKCS7 *)get_body_sparep_data(body->sparep)
 		: NULL;
     if(p7){
 
@@ -898,6 +900,7 @@ smime_config_init_display(struct pine *ps, CONF_S **ctmp, CONF_S **first_line)
 
     for(i = 0; i < sizeof(tmp) && i < (ps->ttyo ? ps->ttyo->screen_cols : sizeof(tmp)); i++)
 	tmp[i] = '-';
+    tmp[i] = '\0';
     new_confline(ctmp);
     (*ctmp)->flags |= CF_NOSELECT;
     (*ctmp)->value = cpystr(tmp);
@@ -987,7 +990,8 @@ smime_config_init_display(struct pine *ps, CONF_S **ctmp, CONF_S **first_line)
 
 #endif /* APPLEKEYCHAIN */
 
-    if(SMHOLDERTYPE(Private) == Keychain
+    if(ps_global->smime
+	&& SMHOLDERTYPE(Private) == Keychain
 	&& SMHOLDERTYPE(Public) == Keychain
 	&& SMHOLDERTYPE(CACert) == Keychain)
 	return;
@@ -1069,7 +1073,8 @@ smime_config_init_display(struct pine *ps, CONF_S **ctmp, CONF_S **first_line)
     (*ctmp)->next	    = NULL;
 }
 
-void display_certificate_information(struct pine *ps, X509 *cert, char *email, WhichCerts ctype, int num)
+void
+display_certificate_information(struct pine *ps, X509 *cert, char *email, WhichCerts ctype, int num)
 {
     STORE_S  *store;
     SCROLL_S  scrollargs;
@@ -1257,7 +1262,7 @@ manage_certificate_info_tool(int cmd, MSGNO_S *msgmap, SCROLL_S *sparms)
      case MC_TRUST: rv = 1; break;
 	default: rv = 0; break;
   }
- return rv;
+  return rv;
 }
 
 
@@ -1286,18 +1291,21 @@ manage_certs_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags)
 					pathdir, MASTERNAME);
 		filename[sizeof(filename)-1] = '\0';
 		rv = import_certificate(ctype, pc, filename);
-		if(our_stat(pathdir, &sbuf) == 0){
-		  if(unlink(filename) < 0)
-		   q_status_message1(SM_ORDER, 0, 2, 
+		if(rv == 1){
+		  ps->keyemptypwd = 0;
+		  if(our_stat(pathdir, &sbuf) == 0){
+		    if(unlink(filename) < 0)
+		      q_status_message1(SM_ORDER, 0, 2, 
 			_("Could not remove private key %s.key"), MASTERNAME);
-		  filename[strlen(filename)-4] = '\0';
-		  strcat(filename, ".crt");
-		  if(unlink(filename) < 0)
-		   q_status_message1(SM_ORDER, 0, 2, 
-			_("Could not remove public certifica %s.crt"), MASTERNAME);
-		  if(rmdir(pathdir) < 0)
-		    q_status_message1(SM_ORDER, 0, 2, 
+		    filename[strlen(filename)-4] = '\0';
+		    strcat(filename, ".crt");
+		    if(unlink(filename) < 0)
+		      q_status_message1(SM_ORDER, 0, 2, 
+			_("Could not remove public certificate %s.crt"), MASTERNAME);
+		    if(rmdir(pathdir) < 0)
+		      q_status_message1(SM_ORDER, 0, 2, 
 		      _("Could not remove temporary directory %s"), pathdir);
+		  }
 		}
 	      }
 	      rv = 10;		/* forces redraw */
@@ -1319,13 +1327,71 @@ manage_certs_tool(struct pine *ps, int cmd, CONF_S **cl, unsigned flags)
 	   break;
 
       case MC_DELETE:
-	   if ((*cl)->d.s.deleted != 0)
-	      q_status_message(SM_ORDER, 1, 3, _("Certificate already deleted"));
-	   else{
-	      (*cl)->d.s.deleted = 1;
-	      rv = 10 + (*cl)->varmem;		/* forces redraw */
-	      mark_cert_deleted(ctype, (*cl)->varmem, 1);
-	      q_status_message(SM_ORDER, 1, 3, _("Certificate marked deleted"));
+	   if(ctype == Password){
+	      EVP_PKEY *key = NULL;
+	      PERSONAL_CERT *pc =  (PERSONAL_CERT *) ps->pwdcert;
+	      RSA *rsa = NULL;
+	      const EVP_CIPHER *enc = NULL;
+	      BIO *out = NULL;
+	      BIO *in = NULL;
+	      char filename[MAXPATH+1];
+	      char passwd[MAILTMPLEN];
+	      char prompt[MAILTMPLEN];
+
+	      if (pc != NULL && pc->key != NULL){
+		  strncpy(prompt, _("Enter password to unlock key: "), sizeof(prompt));
+		  prompt[sizeof(prompt)-1] = '\0';
+		  passwd[0] = '\0';
+
+		  rv = alpine_get_password(prompt, passwd, sizeof(passwd));
+
+		  if(rv == 1)
+		     q_status_message(SM_ORDER, 1, 3, _("Password deletion cancelled"));
+		  else if(rv == 0){
+		     snprintf(filename, sizeof(filename), "%s/%s.key", ps->pwdcertdir, pc->name);
+		     filename[sizeof(filename)-1] = '\0';
+		     if((in = BIO_new_file(filename, "r")) != NULL){
+			key = PEM_read_bio_PrivateKey(in, NULL, NULL, passwd);
+			if(key != NULL){
+			  if((rsa = EVP_PKEY_get1_RSA(key)) != NULL
+			     && (out = BIO_new(BIO_s_file())) != NULL
+			     && BIO_write_filename(out, filename) > 0
+			     && PEM_write_bio_RSAPrivateKey(out, rsa, enc, NULL, 0, NULL, passwd) > 0){
+			     q_status_message(SM_ORDER, 1, 3, _("Password Removed from private key"));
+			     ps->keyemptypwd = 1;
+			  }
+			  else
+			     rv = 1;
+			}
+			else{
+			  rv = 1;
+			  q_status_message(SM_ORDER, 1, 3, _("Failed to unlock private key"));
+			}
+			BIO_free(in);
+		     }
+		     else
+			rv = 1;
+		  }
+		  if(rv == 1)
+		     q_status_message(SM_ORDER, 1, 3, _("Failed to remove password from private key"));
+		  rv += 10;		/* forces redraw */
+		  if(out != NULL)
+		    BIO_free_all(out);
+		  if(rsa != NULL)
+		    RSA_free(rsa);
+		  if(key != NULL)
+		    EVP_PKEY_free(key);
+	      }
+	   }
+	   else {
+	      if ((*cl)->d.s.deleted != 0)
+	         q_status_message(SM_ORDER, 1, 3, _("Certificate already deleted"));
+	      else{
+	         (*cl)->d.s.deleted = 1;
+	         rv = 10 + (*cl)->varmem;		/* forces redraw */
+	         mark_cert_deleted(ctype, (*cl)->varmem, 1);
+	         q_status_message(SM_ORDER, 1, 3, _("Certificate marked deleted"));
+	      }
 	   }
 	   break;
 
@@ -1395,17 +1461,20 @@ smime_setup_size(char **s, size_t buflen, size_t n)
    snprintf(t, buflen-3, "%zu.%zu", n, n);
    t += strlen(t);
    *t++ = 's';
+   *t = '\0';
    *s = t;
 }
 
 #ifdef PASSFILE
-void manage_password_file_certificates(struct pine *ps)
+void
+manage_password_file_certificates(struct pine *ps)
 {
     OPT_SCREEN_S    screen;
     int             readonly_warning = 0, rv = 10, fline, state = 0;
 
     dprint((9, "manage_password_file_certificates"));
     ps->next_screen = SCREEN_FUN_NULL;
+    ps->keyemptypwd = 0;	/* just in case */
 
     do {
       CONF_S *ctmp = NULL, *first_line = NULL;
@@ -1422,6 +1491,7 @@ void manage_password_file_certificates(struct pine *ps)
 
       memset(&screen, 0, sizeof(screen));
       screen.deferred_ro_warning = readonly_warning;
+
       rv = conf_scroll_screen(ps, &screen, first_line,
 			       _("MANAGE PASSWORD FILE CERTS"),
 			      /* TRANSLATORS: Print something1 using something2.
@@ -1430,6 +1500,7 @@ void manage_password_file_certificates(struct pine *ps)
     } while (rv != 0);
 
     ps->mangled_screen = 1;
+    ps->keyemptypwd = 0;	/* reset this so it will not confuse other routines */
     smime_reinit();
 }
 
@@ -1444,9 +1515,12 @@ smime_manage_password_file_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **f
     CertList *cl;
     int	  i;
     void  *pwdcert = NULL;	/* this is our current password file */
-    PERSONAL_CERT *pc;
     X509_LOOKUP *lookup = NULL;
     X509_STORE  *store  = NULL;
+    char filename[MAXPATH+1];
+    BIO *in = NULL;
+    EVP_PKEY *key = NULL;
+    PERSONAL_CERT *pc;
 
     if(*state == 0){		/* first time around? */
       setup_pwdcert(&pwdcert);
@@ -1459,10 +1533,21 @@ smime_manage_password_file_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **f
     }
 
     pc = (PERSONAL_CERT *) ps_global->pwdcert;
+    snprintf(filename, sizeof(filename), "%s/%s.key", ps->pwdcertdir, pc->name);
+    filename[sizeof(filename)-1] = '\0';
+    if((in = BIO_new_file(filename, "r")) != NULL
+	&& (key = PEM_read_bio_PrivateKey(in, NULL, NULL, "")) != NULL)
+	ps->keyemptypwd = 1;
+    if(in != NULL)
+	BIO_free(in);
+    if(key != NULL)
+	EVP_PKEY_free(key);
+
     ps->pwdcertlist = cl = smime_X509_to_cert_info(X509_dup(pc->cert), pc->name);
 
     for(i = 0; i < sizeof(tmp) && i < (ps->ttyo ? ps->ttyo->screen_cols : sizeof(tmp)); i++)
 	tmp[i] = '-';
+    tmp[i] = '\0';
 
     new_confline(ctmp);
     (*ctmp)->flags |= CF_NOSELECT;
@@ -1495,8 +1580,10 @@ smime_manage_password_file_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **f
       smime_setup_size(&t, sizeof(u) - strlen(t), e);
       smime_setup_size(&t, sizeof(u) - strlen(t), df);
       *t++ = ' ';	/* leave an extra space between dates */
+      *t = '\0';	/* make valgrind happy */
       smime_setup_size(&t, sizeof(u) - strlen(t), dt);
       *t++ = ' ';	/* and another space between date and md5 sum */
+      *t = '\0';	/* make valgrind happy again */
       smime_setup_size(&t, sizeof(u) - strlen(t), md5);
       *t = '\0';	/* tie off */
 
@@ -1531,9 +1618,10 @@ smime_manage_password_file_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **f
       (*ctmp)->d.s.deleted = 0;
       (*ctmp)->help	   = h_config_smime_password_file_certificates;
       (*ctmp)->tool        = manage_certs_tool;
-      (*ctmp)->keymenu     = &config_smime_manage_view_cert_keymenu;
+      (*ctmp)->keymenu     = ps->keyemptypwd == 0
+			     ? &config_smime_manage_view_cert_keymenu
+			     : &config_smime_manage_view_cert_keymenu_no_delete;
       (*ctmp)->varmem      = 0;
-      (*ctmp)->help	   = h_config_smime_manage_public_menu;
       strncpy((*ctmp)->d.s.address, cl->name, sizeof((*ctmp)->d.s.address));
       (*ctmp)->d.s.address[sizeof((*ctmp)->d.s.address) - 1] = '\0';
       snprintf(tmp, sizeof(tmp), u,
@@ -1541,13 +1629,13 @@ smime_manage_password_file_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **f
 			cl->name, 
 			DATEFROMCERT(cl), DATETOCERT(cl), MD5CERT(cl));
       (*ctmp)->value      = cpystr(tmp);
-
-   }
+    }
 }
 #endif /* PASSFILE */
 
 
-void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line, WhichCerts ctype, int fline)
+void
+smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line, WhichCerts ctype, int fline)
 {
     char            tmp[200];
     char	   *ext;
@@ -1564,6 +1652,8 @@ void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line
 
     for(i = 0; i < sizeof(tmp) && i < (ps->ttyo ? ps->ttyo->screen_cols : sizeof(tmp)); i++)
 	tmp[i] = '-';
+    tmp[i] = '\0';
+
     new_confline(ctmp);
     (*ctmp)->flags |= CF_NOSELECT;
     (*ctmp)->value = cpystr(tmp);
@@ -1578,7 +1668,9 @@ void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line
     (*ctmp)->value = cpystr(tmp);
 
     for(i = 0; i < sizeof(tmp) && i < (ps->ttyo ? ps->ttyo->screen_cols : sizeof(tmp)); i++)
-    tmp[i] = '-';
+       tmp[i] = '-';
+    tmp[i] = '\0';
+
     new_confline(ctmp);
     (*ctmp)->flags |= CF_NOSELECT;
     (*ctmp)->value = cpystr(tmp);
@@ -1609,8 +1701,10 @@ void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line
       smime_setup_size(&t, sizeof(u) - strlen(t), e);
       smime_setup_size(&t, sizeof(u) - strlen(t), df);
       *t++ = ' ';	/* leave an extra space between dates */
+      *t = '\0';	/* make valgrind happy */
       smime_setup_size(&t, sizeof(u) - strlen(t), dt);
       *t++ = ' ';	/* and another space between date and md5 sum */
+      *t = '\0';	/* make valgrind happy again */
       smime_setup_size(&t, sizeof(u) - strlen(t), md5);
       *t = '\0';	/* tie off */
 
@@ -1653,7 +1747,8 @@ void smime_manage_certs_init(struct pine *ps, CONF_S **ctmp, CONF_S **first_line
     }
 }
 
-void manage_certificates(struct pine *ps, WhichCerts ctype)
+void
+manage_certificates(struct pine *ps, WhichCerts ctype)
 {
     OPT_SCREEN_S    screen;
     int             readonly_warning = 0, rv = 10, fline;
@@ -1922,8 +2017,7 @@ save_smime_config_vars(struct pine *ps)
 
     vsave = (SAVED_CONFIG_S *)fs_get((V_LAST_VAR+1)*sizeof(SAVED_CONFIG_S));
     memset((void *)vsave, 0, (V_LAST_VAR+1)*sizeof(SAVED_CONFIG_S));
-    v = vsave;
-    for(vreal = ps->vars; vreal->name; vreal++,v++){
+    for(v = vsave, vreal = ps->vars; vreal->name; vreal++,v++){
 	if(!(smime_related_var(ps, vreal) || vreal==&ps->vars[V_FEATURE_LIST]))
 	  continue;
 	
@@ -1938,7 +2032,7 @@ save_smime_config_vars(struct pine *ps)
 		while(list[n])
 		  n++;
 
-		v->saved_user_val.l = (char **)fs_get((n+1) * sizeof(char *));
+		v->saved_user_val.l = (char **)fs_get((n+1)*sizeof(char *));
 		memset((void *)v->saved_user_val.l, 0, (n+1)*sizeof(char *));
 		for(i = 0; i < n; i++)
 		  v->saved_user_val.l[i] = cpystr(list[i]);
@@ -1964,7 +2058,7 @@ free_saved_smime_config(struct pine *ps, SAVED_CONFIG_S **vsavep)
 
     if(vsavep && *vsavep){
 	for(v = *vsavep, vreal = ps->vars; vreal->name; vreal++,v++){
-	    if(!(smime_related_var(ps, vreal)))
+	    if(!(smime_related_var(ps, vreal) || vreal==&ps->vars[V_FEATURE_LIST]))
 	      continue;
 	    
 	    if(vreal->is_list){  /* free saved_user_val.l */
