@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: status.c 840 2007-12-01 01:34:49Z hubert@u.washingto
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2017 Eduardo Chappa
+ * Copyright 2013-2020 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -229,7 +229,13 @@ pause_for_and_dq_cur_msg(void)
 
 	if((w = status_message_remaining_nolock()) != 0){
 	    delay_cmd_cue(1);
-	    sleep(w);
+	    /* protect user from changes in the clock. If the clock
+	     * changes during this process (for example, going from 
+	     * Standard time to Delayed time) this may result in big 
+	     * values for w. In those cases, reset w.
+	     */
+	    w = (w > 0 ? (w > 5 ? 5 : w) : 0);
+	    if(ps_global && !ps_global->initial_cmds) sleep(w);
 	    delay_cmd_cue(0);
 	}
 
@@ -249,10 +255,10 @@ pause_for_and_mark_specific_msg(SMQ_T *msg)
 	int w;
 
 	w = (int) (displayed_time - time(0)) + msg->min_display_time;
-	w = (w > 0) ? w : 0;
+	w = (w > 0) ? (w > 5 ? 5 : w): 0;
 	if(w){
 	    delay_cmd_cue(1);
-	    sleep(w);
+	    if (ps_global && !ps_global->initial_cmds) sleep(w);
 	    delay_cmd_cue(0);
 	}
 
@@ -1060,7 +1066,8 @@ output_message(SMQ_T *mq_entry, int ding)
 	      MoveCursor(ps_global->ttyo->screen_rows-FOOTER_ROWS(ps_global), 0);
 
 	    fflush(stdout);
-	    sleep(ps_global->status_msg_delay);
+	    if (ps_global && !ps_global->initial_cmds)
+		sleep(ps_global->status_msg_delay);
 	}
 
 	mq_entry->shown = 1;
@@ -1100,7 +1107,7 @@ output_message_modal(SMQ_T *mq_entry, int ding)
 	char	 *p, *q, *s, *t;
 	SCROLL_S  sargs;
 	
-	/* Count the number of modal messsages and add up their lengths. */
+	/* Count the number of modal messages and add up their lengths. */
 	for(m = mq_entry->next; m != mq_entry; m = m->next)
 	  if((m->flags & SM_MODAL) && !m->shown){
 	      i++;
@@ -1280,3 +1287,23 @@ modal_bogus_input(UCS ch)
     Writechar(BELL, 0);
     return(0);
 }
+
+void
+free_message_queue(void)
+{
+   SMQ_T *m, *mnext;
+
+   if(message_queue == NULL)
+     return;
+
+    m = message_queue;
+    do{
+      if(m->text)
+        fs_give((void **) &m->text);
+     
+      mnext = m->next;
+      fs_give((void **) &m);
+      m = mnext;
+    } while(m != message_queue);
+    message_queue = NULL;
+}
