@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: titlebar.c 1075 2008-06-04 00:19:39Z hubert@u.washin
 
 /*
  * ========================================================================
- * Copyright 2013-2017 Eduardo Chappa
+ * Copyright 2013-2020 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -1261,3 +1261,12 @@ newmail_check_point_cue(int onoroff)
       mswin_setcursor (MSWIN_CURSOR_ARROW);
 #endif
 }
+
+void
+free_titlebar_globals(void)
+{
+  if(as_fname) fs_give((void **) &as_fname);
+  if(as_cname) fs_give((void **) &as_cname);
+  if(as.folder_name) fs_give((void **)&as.folder_name);
+  if(as.context_name) fs_give((void **)&as.context_name);
+}
