@@ -15,18 +15,24 @@
 #ifndef ALPINE_XOAUTH2_INCLUDED
 #define ALPINE_XOAUTH2_INCLUDED
 
-typedef struct xoauth_default_s {
-   unsigned char *name;
-   char *client_id;
-   char *client_secret;
-} XOAUTH2_INFO_S;
-
 #define GMAIL_NAME "Gmail"
 #define GMAIL_ID "624395471329-0qee3goofj7kbl7hsukou3rqq0igntv1.apps.googleusercontent.com"
 #define GMAIL_SECRET "vwnqVJQrJZpR6JilCfAN5nY7"
+#define GMAIL_TENANT NULL
 
 #define OUTLOOK_NAME "Outlook"
 #define OUTLOOK_ID   "f21dcaf2-8020-469b-8135-343bfc35d046"
-#define OUTLOOK_SECRET NULL
+#define OUTLOOK_SECRET "Tk-DAcEi13-FeSsY_Ja4Y.-MyL66I.wIPt"
+#define OUTLOOK_TENANT "common"
+
+#define YANDEX_NAME   "Yandex"
+#define YANDEX_ID     "393578fee26f47858023cf59681882a7"
+#define YANDEX_SECRET "7304c4993583498f8ab63e2f21ad6960"
+#define YANDEX_TENANT NULL
+
+#define YAHOO_NAME   "Yahoo!"
+#define YAHOO_ID     "dj0yJmk9RTdyZEQ2TWxGMzV6JmQ9WVdrOWVrTllZbGgyV2tjbWNHbzlNQT09JnM9Y29uc3VtZXJzZWNyZXQmc3Y9MCZ4PTli"
+#define YAHOO_SECRET "3a8b12d51c09b0a5c0733c36d04cd3c69e33baef"
+#define YAHOO_TENANT NULL
 
 #endif /* ALPINE_XOAUTH2_INCLUDED */
