@@ -196,8 +196,12 @@
 #define SET_SSLCAPATH (long) 232
 #define GET_SSLCAFILE (long) 233
 #define SET_SSLCAFILE (long) 234
-#define GET_RESTRICTIONS (long) 235
-#define SET_RESTRICTIONS (long) 236
+#define GET_SSLAPPCAPATH (long) 235
+#define SET_SSLAPPCAPATH (long) 236
+#define GET_SSLAPPCAFILE (long) 237
+#define SET_SSLAPPCAFILE (long) 238
+#define GET_RESTRICTIONS (long) 239
+#define SET_RESTRICTIONS (long) 240
 
 	/* 3xx: TCP/IP */
 #define GET_OPENTIMEOUT (long) 300
@@ -1150,6 +1154,10 @@ typedef struct mail_stream {
   void *local;			/* pointer to driver local data */
   char *mailbox;		/* mailbox name (canonicalized) */
   char *original_mailbox;	/* mailbox name (non-canonicalized) */
+  struct {
+    char *name;			/* AUTHENTICATE method */
+    unsigned long expiration;	/* expiration time for authentication */
+  } auth;
   unsigned short use;		/* stream use count */
   unsigned short sequence;	/* stream sequence */
   unsigned int inbox : 1;	/* stream open on an INBOX */
@@ -1578,6 +1586,8 @@ DRIVER {
   long (*append) (MAILSTREAM *stream,char *mailbox,append_t af,void *data);
 				/* garbage collect stream */
   void (*gc) (MAILSTREAM *stream,long gcflags);
+				/* renew stream */
+  long (*renew) (MAILSTREAM *stream, MAILSTREAM *m);
 };
 
 
@@ -1702,6 +1712,7 @@ long mail_rename (MAILSTREAM *stream,char *old,char *newname);
 char *mail_utf7_valid (char *mailbox);
 long mail_status (MAILSTREAM *stream,char *mbx,long flags);
 long mail_status_default (MAILSTREAM *stream,char *mbx,long flags);
+long mail_renew_stream (MAILSTREAM *stream);
 MAILSTREAM *mail_open (MAILSTREAM *stream,char *name,long options);
 MAILSTREAM *mail_open_work (DRIVER *d,MAILSTREAM *stream,char *name,
 			    long options);
@@ -1879,7 +1890,7 @@ void auth_link (AUTHENTICATOR *auth);
 char *mail_auth (char *mechanism,authresponse_t resp,int argc,char *argv[]);
 AUTHENTICATOR *mail_lookup_auth (unsigned long i);
 unsigned int mail_lookup_auth_name (char *mechanism,long flags);
-
+void free_id(IDLIST **);
 NETSTREAM *net_open (NETMBX *mb,NETDRIVER *dv,unsigned long port,
 		     NETDRIVER *ssld,char *ssls,unsigned long sslp);
 NETSTREAM *net_open_work (NETDRIVER *dv,char *host,char *service,
@@ -1929,6 +1940,7 @@ int PFLUSH (void);
 
 typedef enum {OA2_Id = 0,
 	      OA2_Secret,
+	      OA2_Tenant,
 	      OA2_Code,
 	      OA2_RefreshToken,
 	      OA2_Scope,
@@ -1974,9 +1986,12 @@ typedef struct oauth2_s {
    OAUTH2_SERVER_METHOD_S server_mthd[OA2_GetEnd];
    OAUTH2_DEVICECODE_S devicecode;
    char *access_token;
+   char *app_id;		/* special id for this server */
+   unsigned hide:1;		/* hide this from user */
    unsigned long expiration;
    unsigned int first_time:1;	/* this is the first time we get credentials for this account */
    unsigned int require_secret:1;	/* this server requires a client-secret */
+   int cancel_refresh_token;	/* ask client to cancel refresh token */
 } OAUTH2_S;
 
 typedef struct deviceproc_s {
@@ -1986,10 +2001,24 @@ typedef struct deviceproc_s {
   char code_wait;		/* code to say keep waiting */
 } OAUTH2_DEVICEPROC_S;
 
+typedef struct xoauth_default_s {
+   unsigned char *name;
+   char *client_id;
+   char *client_secret;
+   char *tenant;
+   char *users;
+   char *flow;
+} XOAUTH2_INFO_S;
+
 /* Supporting external functions for XOAUTH2 and OAUTHBEARER */
 typedef char *(*oauth2getaccesscode_t) (unsigned char *, char *, OAUTH2_S *, int *);
-typedef void (*oauth2clientinfo_t)(unsigned char *name, char **id, char **secret);
+typedef XOAUTH2_INFO_S *(*oauth2clientinfo_t)(unsigned char *name, char *user);
 typedef void (*oauth2deviceinfo_t)(OAUTH2_S *, char *method);
 void mm_login_oauth2_c_client_method (NETMBX *, char *, char *, OAUTH2_S *, unsigned long, int *);
 char *oauth2_generate_state(void);
 void oauth2deviceinfo_get_accesscode(void *, void *);
+XOAUTH2_INFO_S *new_xoauth2_info(void);
+void    free_xoauth2_info(XOAUTH2_INFO_S **);
+XOAUTH2_INFO_S *copy_xoauth2_info(XOAUTH2_INFO_S *);
+char *oauth2_generate_state(void);
+void renew_accesstoken(MAILSTREAM *);
