@@ -23,7 +23,7 @@
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
  */
-#if defined(ENABLE_WINDOWS_LIBRESSL) && (defined(W32BITSBUILD) || defined(WXPBUILD))
+#if defined(ENABLE_WINDOWS_UNIXSSL) && defined(WXPBUILD)
 #include "ssl_libressl.c"
 #else
 #include "ssl_win.c"
