@@ -3,7 +3,7 @@
  *
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2017 Eduardo Chappa
+ * Copyright 2013-2020 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -134,6 +134,7 @@ extern	int mlreplyd_utf8(char *, char *, int, int, EXTRAKEYS *);
 extern	int mlreplyd(UCS *, UCS *, int, int, EXTRAKEYS *);
 extern	int mlwrite_utf8(char *, void *);
 extern	int mlwrite(UCS *, void *);
+extern	void emlwwrite(char *, EML *);
 extern	void emlwrite(char *, EML *);
 extern	void emlwrite_ucs4(UCS *, EML *);
 extern	void unknown_command(UCS);
@@ -208,6 +209,7 @@ extern	int ctrlg(int, int);
 extern	int rdonly(void);
 extern	int pico_help(char **, char *, int);
 extern	void zotedit(void);
+extern  void free_pico_module_globals(void);
 #ifdef	_WINDOWS
 int	composer_file_drop(int, int, char *);
 int	pico_cursor(int, long);
