@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: file.c 1082 2008-06-12 18:39:50Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2017 Eduardo Chappa
+ * Copyright 2013-2020 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -304,8 +304,8 @@ insfile(int f, int n)
 		    char tfname[NLINE];
 
 		    if(gmode&MDSCUR){
-			emlwrite(
-			      _("\007Restricted mode disallows uploaded command"),
+			emlwwrite(
+			      _("Restricted mode disallows uploaded command"),
 			      NULL);
 			return(0);
 		    }
@@ -557,7 +557,7 @@ readin(char fname[],		/* name of file to read */
  * Update the remembered file name and clear the
  * buffer changed flag. This handling of file names
  * is different from the earlier versions, and
- * is more compatable with Gosling EMACS than
+ * is more compatible with Gosling EMACS than
  * with ITS EMACS. Bound to "C-X C-W".
  */
 int
@@ -795,7 +795,7 @@ filewrite(int f, int n)
 
 /*
  * Save the contents of the current
- * buffer in its associatd file. No nothing
+ * buffer in its associated file. No nothing
  * if nothing has changed (this may be a bug, not a
  * feature). Error if there is no remembered file
  * name for the buffer. Bound to "C-X C-S". May
