@@ -124,9 +124,11 @@ typedef	enum {    V_PERSONAL_NAME = 0
 		, V_DOWNLOAD_CMD_PREFIX
 		, V_MAILCAP_PATH
 		, V_MIMETYPE_PATH
-#if !defined(_WINDOWS) || defined(WINDOWS_LIBRESSL_CERTS)
+#if !defined(_WINDOWS) || defined(WINDOWS_UNIXSSL_CERTS)
 		, V_SSLCAPATH
 		, V_SSLCAFILE
+		, V_USERSSLCAPATH
+		, V_USERSSLCAFILE
 #endif
 		, V_BROWSER
 		, V_HISTORY
@@ -174,9 +176,7 @@ typedef	enum {    V_PERSONAL_NAME = 0
 		, V_NEW_VER_QUELL
 		, V_DISABLE_DRIVERS
 		, V_DISABLE_AUTHS
-#ifdef DF_ENCRYPTION_RANGE
 		, V_ENCRYPTION_RANGE
-#endif
 		, V_REMOTE_ABOOK_METADATA
 		, V_REMOTE_ABOOK_HISTORY
 		, V_REMOTE_ABOOK_VALIDITY
