@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: ldap.c 1204 2009-02-02 19:54:23Z hubert@u.washington
 
 /*
  * ========================================================================
- * Copyright 2013-2017 Eduardo Chappa
+ * Copyright 2013-2020 Eduardo Chappa
  * Copyright 2006-2008 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -62,7 +62,7 @@ int              our_ldap_set_lderrno(LDAP *, int, char *, char *);
  * Args  string -- the string to use in the lookup
  *
  * Returns     NULL -- lookup failed
- *          Address -- A single address is returned if lookup was successfull.
+ *          Address -- A single address is returned if lookup was successful.
  */
 ADDRESS *
 wp_lookups(char *string, WP_ERR_S *wp_err, int recursing)
@@ -398,7 +398,7 @@ ldap_lookup(LDAP_SERV_S *info, char *string, CUSTOM_FILT_S *cust,
 {
     char     ebuf[900];
     char     buf[900];
-    char    *serv, *base, *serv_errstr;
+    char    *serv, *base, *serv_errstr, *s, *t;
     char    *mailattr, *snattr, *gnattr, *cnattr;
     int      we_cancel = 0, we_turned_on = 0;
     LDAP_SERV_RES_S *serv_res = NULL;
@@ -526,9 +526,28 @@ ldap_lookup(LDAP_SERV_S *info, char *string, CUSTOM_FILT_S *cust,
     }
 #endif /* SMIME_SSLCERTS */
 
-    snprintf(tmp_20k_buf, SIZEOF_20KBUF, "%s://%s:%d", 
-		info->ldaps ? "ldaps" : "ldap", serv, info->port);
-    tmp_20k_buf[SIZEOF_20KBUF-1] = '\0';
+    tmp_20k_buf[0] = '\0';
+    s = serv;
+    do {
+       if ((t = strchr(s, ' ')) != NULL) *t = '\0';
+       snprintf(tmp_20k_buf + strlen(tmp_20k_buf),
+		SIZEOF_20KBUF - strlen(tmp_20k_buf), "%s://%s",
+		info->ldaps ? "ldaps" : "ldap", s);
+       if (strchr(s, ':') == NULL){
+          snprintf(tmp_20k_buf + strlen(tmp_20k_buf),
+		SIZEOF_20KBUF - strlen(tmp_20k_buf),
+	        "%s%d", ":", info->port);
+       }
+       if(t != NULL){
+	  *t = ' ';
+	  for( ; *t == ' '; t++);
+          snprintf(tmp_20k_buf + strlen(tmp_20k_buf),
+		SIZEOF_20KBUF - strlen(tmp_20k_buf), "%s", " ");
+       }
+       s = t;
+    } while (s != NULL);
+
+    tmp_20k_buf[SIZEOF_20KBUF - 1] = '\0';
 
     if(ldap_initialize(&ld, tmp_20k_buf) != LDAP_SUCCESS) 
 #endif
@@ -556,7 +575,7 @@ ldap_lookup(LDAP_SERV_S *info, char *string, CUSTOM_FILT_S *cust,
     }
     else if(!ps_global->intr_pending){
       int proto = 3, tlsmustbail = 0;
-      char pwd[NETMAXPASSWD], user[NETMAXUSER];
+      char *pwd = NULL, user[NETMAXUSER];
 #ifdef _WINDOWS
       char *passwd = NULL;
 #else
@@ -634,7 +653,7 @@ try_password_again:
 
 	  if(!tlsmustbail){
 	      snprintf(pmt, sizeof(pmt), "  %s", (info->nick && *info->nick) ? info->nick : serv);
-	      mm_login_work(&mb, user, pwd, pwdtrial, pmt, info->binddn);
+	      mm_login_work(&mb, user, &pwd, pwdtrial, pmt, info->binddn);
 	      if(pwd && pwd[0])
 #ifdef _WINDOWS
 		passwd = pwd;
@@ -1172,6 +1191,8 @@ try_password_again:
 	  }
 	}
       }
+      if(pwd)
+	fs_give((void **) &pwd);
     }
 
     if(we_cancel)
