@@ -4,7 +4,7 @@ static char rcsid[] = "$Id: mailcmd.c 1142 2008-08-13 17:22:21Z hubert@u.washing
 
 /*
  * ========================================================================
- * Copyright 2013-2017 Eduardo Chappa
+ * Copyright 2013-2020 Eduardo Chappa
  * Copyright 2006-2007 University of Washington
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -1663,7 +1663,7 @@ expunge_and_close(MAILSTREAM *stream, char **final_msg, long unsigned int flags)
         }
 
 	/*
-	 * Make darn sure any mm_log fallout caused above get's seen...
+	 * Make darn sure any mm_log fallout caused above gets seen...
 	 */
 	if(!no_close){
 	    flush_status_messages(1);
@@ -2342,6 +2342,12 @@ agg_text_select(MAILSTREAM *stream, MSGNO_S *msgmap, char type, char *namehdr,
 
   if(!mepgm)
     switch(type){
+      case 'g' :				/* X-GM-EXT-1 */
+	pgm->x_gm_ext1 = mail_newstringlist();
+	pgm->x_gm_ext1->text.data = (unsigned char *) cpystr(namehdr);
+	pgm->x_gm_ext1->text.size = strlen(namehdr);
+      break;
+
       case 'h' :	                        /* Any header */
         pgm->header = mail_newsearchheader (namehdr, sstring);
       break;
