@@ -5,7 +5,7 @@ static char rcsid[] = "$Id: msgno.c 854 2007-12-07 17:44:43Z hubert@u.washington
 /*
  * ========================================================================
  * Copyright 2006-2007 University of Washington
- * Copyright 2013-2017 Eduardo Chappa
+ * Copyright 2013-2020 Eduardo Chappa
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -864,7 +864,7 @@ calculate_some_scores(MAILSTREAM *stream, SEARCHSET *searchset, int no_fetch)
 		 * we will actually have fetched this with
 		 * the envelope. Just making sure.
 		 */
-		if(hdrtok && no_fetch){
+		if(no_fetch){
 		    error++;
 		    break;
 		}
@@ -937,6 +937,9 @@ free_pine_elt(void **sparep)
 	if((*peltp)->pthrd)
 	  fs_give((void **) &(*peltp)->pthrd);
 
+	if((*peltp)->firsttext)
+	  fs_give((void **) &(*peltp)->firsttext);
+
 	if((*peltp)->ice)
 	  free_ice(&(*peltp)->ice);
 
